# Proc.pm - Proc generic interface file for generator (IPTables Firewall Configuration).
# Created by James A. Pattie.  Copyright (c) 2001-2004, Xperience, Inc.

package PCXFireWall::Proc;

use strict;
use vars qw ($AUTOLOAD @ISA @EXPORT $VERSION);

require Exporter;
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = "2.23";

sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  $self->{modules} = {};
  $self->{knownModules} = "Filter|Nat";  # regular expression format

  # now create the Filter and Nat and their Pre and Post entries
  foreach my $module ("Filter", "Nat")
  {
    $self->{modules}->{$module} = {};
    foreach my $when ("Pre", "Post")
    {
      $self->{modules}->{$module}->{$when} = [];
    }
  }

  return $self;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# sysNetIPv4(file, value, module, required, when)
#   required: file, value
#   optional: module, required, when
#   directory: /proc/sys/net/ipv4
#   summary:  Generates the shell script code to update the specified
#             file with the specified value.  The code is generated
#             and stored in the Pre or Post array for the module.
#             If module is not specified, it defaults to Filter.
#             If when is not specified, it defaults to Pre.
#             If required is not specified, it defaults to false.
#             If the file must exist then the code is written to
#             generate an error and then stop running otherwise the
#             code will just generate an error if the file is not
#             found when it executes.
#   valid values:
#             module (Filter, Nat)
#             required (true, false)
#             when (Pre, Post)
sub sysNetIPv4
{
  my $self = shift;
  my %args = ( file => "", value => "", module => "Filter", required => "false", when => "Pre", @_ );
  my $errStr = "PCXFirewall::Proc->sysNetIPv4()  - Error!\n";
  my $file = $args{file};
  my $value = $args{value};
  my $module = $args{module};
  my $required = $args{required};
  my $when = $args{when};

  my $errorStr = "";
  if (length $file == 0)
  {
    $errorStr .= "You must specify the file to work with!\n";
  }
  if (length $value == 0)
  {
    $errorStr .= "You must specify a value to write to the file!\n";
  }
  if ($module !~ /^($self->{knownModules})$/)
  {
    $errorStr .= "module = '$module' is invalid!\n";
  }
  if ($required !~ /^(true|false)$/)
  {
    $errorStr .= "required = '$required' is invalid!\n";
  }
  if ($when !~ /^(Pre|Post)$/)
  {
    $errorStr .= "when = '$when' is invalid!\n";
  }
  if ($file eq "tcp_fin_timeout" && ($value < 10 || $value > 300))
  {
    $errorStr .= "tcp_fin_timeout of '$value' is invalid!  Range is 10-300.\n";
  }
  if ($file eq "tcp_keepalive" && ($value < 900 || $value > 28800))
  {
    $errorStr .= "tcp_keepalive of '$value' is invalid!  Range is 900-28800.\n";
  }
  if ($file eq "ip_conntrack_max")
  {
    if ($value < 256 || $value > 65528)
    {
      $errorStr .= "ip_conntrack_max of '$value' is invalid!  Range is 256-65528.\n";
    }
    elsif ($module ne "Nat")
    {
      $errorStr .= "ip_conntrack_max can only be used with the Nat module!\n";
    }
  }
  if ($file eq "ip_forward")
  {
    if ($value !~ /^(0|1)$/)
    {
      $errorStr .= "ip_forward can only have 0 or 1 set!\n";
    }
    elsif ($module ne "Nat")
    {
      $errorStr .= "ip_forward can only be used with the Nat module!\n";
    }
  }
  if ($file eq "ip_dynaddr" && $value !~ /^(0|1)$/)
  {
    $errorStr .= "ip_dynaddr can only have 0 or 1 set!\n";
  }
  if ($file eq "tcp_syncookies" && $value !~ /^(0|1)$/)
  {
    $errorStr .= "tcp_syncookies can only have 0 or 1 set!\n";
  }

  if (length $errorStr > 0)
  {
    die $errStr . $errorStr;
  }

  # build the shell script code to be run.
  my $command = "";
  if ($required eq "true")
  {
    $command .= "  sysNetIPv4Required \"$file\" \"$value\"\n";
  }
  else
  {
    $command .= "  sysNetIPv4 \"$file\" \"$value\"\n";
  }

  # now put it in the corresponding modules Pre or Post array
  push @{$self->{modules}->{$module}->{$when}}, $command;
}

# sysNetIPv4Conf(file, value, module, required, when, iface)
#   required: file, value
#   optional: module, required, when, iface
#   directory: /proc/sys/net/ipv4/conf
#   summary: See sysNetIPv4 for info on everything but iface.
#           iface allows you to specify the interface to work with.
#   valid values:
#           iface (all or any valid interface defined on the system)
sub sysNetIPv4Conf
{
  my $self = shift;
  my %args = ( file => "", value => "", module => "Filter", required => "false", when => "Pre", iface => "all", @_ );
  my $errStr = "PCXFirewall::Proc->sysNetIPv4()  - Error!\n";
  my $file = $args{file};
  my $value = $args{value};
  my $module = $args{module};
  my $required = $args{required};
  my $when = $args{when};
  my $iface = $args{iface};

  my $errorStr = "";
  if (length $file == 0)
  {
    $errorStr .= "You must specify the file to work with!\n";
  }
  if (length $value == 0)
  {
    $errorStr .= "You must specify a value to write to the file!\n";
  }
  if ($module !~ /^($self->{knownModules})$/)
  {
    $errorStr .= "module = '$module' is invalid!\n";
  }
  if ($required !~ /^(true|false)$/)
  {
    $errorStr .= "required = '$required' is invalid!\n";
  }
  if ($when !~ /^(Pre|Post)$/)
  {
    $errorStr .= "when = '$when' is invalid!\n";
  }
  if (length $iface == 0)
  {
    $errorStr .= "iface must be specified!\n";
  }

  if ($file eq "rp_filter" && $value !~ /^(0|1)$/)
  {
    $errorStr .= "rp_filter can only have 0 or 1 set!\n";
  }
  if ($file eq "accept_source_route" && $value !~ /^(0|1)$/)
  {
    $errorStr .= "accept_source_route can only have 0 or 1 set!\n";
  }
  if ($file eq "accept_redirects" && $value !~ /^(0|1)$/)
  {
    $errorStr .= "accept_redirects can only have 0 or 1 set!\n";
  }
  if ($file eq "log_martians" && $value !~ /^(0|1)$/)
  {
    $errorStr .= "log_martians can only have 0 or 1 set!\n";
  }

  if (length $errorStr > 0)
  {
    die $errStr . $errorStr;
  }

  # build the shell script code to be run.
  my $command = "";
  if ($required eq "true")
  {
    $command .= "  sysNetIPv4ConfRequired \"$iface\" \"$file\" \"$value\"\n";
  }
  else
  {
    $command .= "  sysNetIPv4Conf \"$iface\" \"$file\" \"$value\"\n";
  }

  # now put it in the corresponding modules Pre or Post array
  push @{$self->{modules}->{$module}->{$when}}, $command;
}

# scalar display(module, when)
#   This routine will write the rules to a string.  This is
#   called by the generator script.
#   module and when determine which branch to display so that the
#   Pre and Post rules get displayed at the correct time.
sub display
{
  my $self = shift;
  my %args = ( module => "Filter", when => "Pre", @_ );
  my $module = $args{module};
  my $when = $args{when};
  my $result = "";

  if ($module !~ /^($self->{knownModules})$/)
  {
    die "(Proc->display) - Error: module = '$module' is invalid!\n";
  }
  if ($when !~ /^(Pre|Post)$/)
  {
    die "(Proc->display) - Error: when = '$when' is invalid!\n";
  }

  $result .= "\n  #\n  # $module Module /proc entries - $when\n  #\n\n";

  # now output all entries we generated.
  foreach my $entry (@{$self->{modules}->{$module}->{$when}})
  {
    $result .= $entry;
  }
  
  return $result;
}

# scalar displayShellFunctions()
#   This routine will output the necessary shell functions to a string.
#   This is called by the generator script.
sub displayShellFunctions
{
  my $self = shift;
  my %args = ( @_ );
  my $result = "";

  $result .= <<"END_OF_CODE";

# \$1 = file, \$2 = value
function sysNetIPv4 ()
{
  if [ -e /proc/sys/net/ipv4/\$1 ]; then
    echo "\$2" > /proc/sys/net/ipv4/\$1
    return 1
  else
    echo "/proc/sys/net/ipv4/\$1 does not exist!"
    return 0
  fi
}

# \$1 = file, \$2 = value
function sysNetIPv4Required ()
{
  sysNetIPv4 \$1 \$2
  if [ \$? -eq 0 ]; then
    exit 1
  fi
}

# \$1 = iface, \$2 = file, \$3 = value
function sysNetIPv4Conf ()
{
  if [ -e /proc/sys/net/ipv4/conf/\$1/\$2 ]; then
    echo "\$3" > /proc/sys/net/ipv4/conf/\$1/\$2
  else
    echo "/proc/sys/net/ipv4/conf/\$1/\$2 does not exist!"
  fi
}

# \$1 = iface, \$2 = file, \$3 = value
function sysNetIPv4ConfRequired ()
{
  sysNetIPv4Conf \$1 \$2 \$3
  if [ \$? -eq 0 ]; then
    exit 1
  fi
}

END_OF_CODE

  return $result;
}

1;
__END__

=head1 NAME

Proc - The PCXFireWall Proc VFS interface.

=head1 SYNOPSIS

  use PCXFireWall::Proc;
  my $proc = PCXFireWall::Proc->new;

  # turn off ip_forward'ing in the Nat module before any nat rules
  $proc->sysNetIPv4(file => "ip_forward", value => "0",
                    module => "Nat", required => "true");

  # you define some Nat rules ...

  # turn ip_forward'ing on now after the nat rules
  $proc->sysNetIPv4(file => "ip_forward", value => "1",
                    module => "Nat", required => "true",
                    when => "Post");

=head1 DESCRIPTION

Proc is the Proc VFS generic interface Object for the PCXFireWall.

=head1 FUNCTIONS

  scalar new(void)
    Creates a new instance of the PCXFireWall::Proc object.
    Initializes the modules hash.

  scalar display(module, when)
    This routine will write the rules to a string.  This is
    called by the generator script.
    module and when determine which branch to display so that the
    Pre and Post rules get displayed at the correct time.

  scalar displayShellFunctions()
    This routine will output the necessary shell functions to a string.
    This is called by the generator script.

  sysNetIPv4(file, value, module, required, when)
    required: file, value
    optional: module, required, when
    directory: /proc/sys/net/ipv4
    summary:  Generates the shell script code to update the specified
              file with the specified value.  The code is generated
              and stored in the Pre or Post array for the module.
              If module is not specified, it defaults to Filter.
              If when is not specified, it defaults to Pre.
              If required is not specified, it defaults to false.
              If the file must exist then the code is written to
              generate an error and then stop running otherwise the
              code will just generate an error if the file is not
              found when it executes.
    valid values:
              module (Filter, Nat)
              required (true, false)
              when (Pre, Post)

  sysNetIPv4Conf(file, value, module, required, when, iface)
    required: file, value
    optional: module, required, when, iface
    directory: /proc/sys/net/ipv4/conf
    summary: See sysNetIPv4 for info on everything but iface.
             iface allows you to specify the interface to work with.
    valid values:
             iface (all or any valid interface defined on the system)

=head1 VARIABLES

  modules - hash of modules (Filter, Nat) with hash entries
            Pre and Post that represent the /proc files
            that should be modified before or after the
            modules rules are output.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin at pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
